/* 
    Webcam Lab

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    (C) 2005 Jason Hunt
    nulluser@gmail.com
   
    file: input.cpp
*/

#include <windows.h>
#include <math.h>
#include <stdio.h>

#include "main.h"
#include "input.h"
#include "display.h"
#include "camera.h"
#include "bitmap.h"
#include "process.h"

#include "move.h"
#include "scan.h"



bool keys[256] = {0}; // Key table

extern bool d_frame;
extern bool running;
extern bool display_fps;

/* Active filters in the pipe line */
extern bool filter_gauss;
extern bool filter_median;
extern bool filter_sharpness;
extern bool filter_edgefast;
extern bool filter_laplace;
extern bool filter_sobel;
extern bool filter_emboss;
extern bool filter_erode;
extern bool filter_threshold;
extern bool filter_tracking;

extern bool filter_average;

extern bool filter_reduceedge;

extern bool filter_comboedge;

extern double threshold_1;
extern double threshold_2;

extern bool fast_capture;

extern bool filter_canny;


extern bool scanning;

/* Called when a key is pressed */
void key_down(WPARAM wParam)
{
    keys[wParam % 256] = true;
   
    if (wParam == VK_ESCAPE) system_shutdown();     // Escape key

    bool mode_change = false;


    // debug filter chain 
    if (keys['V'])
    {
        threshold_1 = 240;
        
        filter_gauss = 1;
        filter_median = 0;
        filter_sharpness = 0;
        filter_edgefast =0;
        filter_laplace = 1;
        filter_sobel = 0;
        filter_emboss = 0;
        filter_threshold = 0;
        filter_erode = 1;
        filter_tracking = 0;
        filter_reduceedge = 0;
        filter_comboedge = 0;
    }


    // unenguage all filters
    if (keys['B'])
    {
    
        filter_gauss = 0;
        filter_median = 0;
        filter_sharpness = 0;
        filter_edgefast = 0;
        filter_laplace = 0;
        filter_sobel = 0;
        filter_emboss = 0;
        filter_threshold = 0;
        filter_erode = 0;
        filter_tracking = 0;
        filter_reduceedge = 0;
        filter_comboedge = 0;
    }


    bool f_change = false;

    if (keys['1']) { f_change = 1; filter_gauss = !filter_gauss; }
    if (keys['2']) { f_change = 1;  filter_median = !filter_median; }
    if (keys['3']) { f_change = 1;  filter_sharpness = !filter_sharpness; }
    if (keys['4']) { f_change = 1;  filter_edgefast = !filter_edgefast; }
    if (keys['5']) { f_change = 1;  filter_laplace = !filter_laplace; }
    if (keys['6']) { f_change = 1;  filter_sobel = !filter_sobel; }
    if (keys['7']) { f_change = 1; filter_emboss = !filter_emboss; }
    if (keys['8']) { f_change = 1; filter_threshold = !filter_threshold; }
    if (keys['9']) { f_change = 1; filter_erode = !filter_erode; }
    if (keys['0']) { f_change = 1; filter_tracking = !filter_tracking; }
    
    if (keys['E']) { f_change = 1; filter_reduceedge = !filter_reduceedge; }
    
    if (keys['Y']) { f_change = 1; filter_average = !filter_average; }
    if (keys['U']) { f_change = 1; filter_comboedge = !filter_comboedge; }
    if (keys['I']) { f_change = 1; filter_canny = !filter_canny; }




    if (f_change) 
        show_filters(); 

    if (wParam == 'F') display_fps = !display_fps;
          
    if (wParam == 'D') 
    {
        d_frame = !d_frame;
        if (d_frame) 
            add_line("Show difference");
        else
            add_line("Show normal");
    }
    
    if (wParam == 'G') 
    {
        fast_capture = !fast_capture;

        if (fast_capture) 
            add_line("Fast Capture");
        else
            add_line("Slow Capture");
            
        clear_buffers();
    }    

    
    if (wParam == 'T') running = !running;

    if (wParam == 'C') setup_camera(true);
    
    if (wParam == 'R') save_to_disk();




    bool t_change = 0;

    if (wParam == 'Q') { threshold_1 -= 2; t_change = 1;}
    if (wParam == 'W') { threshold_1 += 2; t_change = 1; }

    if (wParam == 'A') { threshold_2 -= 1; t_change = 1;}
    if (wParam == 'S') { threshold_2 += 1; t_change = 1; }

    if (t_change)
    {
        // Clamp
        if (threshold_1 < 0) threshold_1 = 0;
        if (threshold_1 > 511) threshold_1 = 511;

        if (threshold_2 < 0) threshold_2 = 0;
        if (threshold_2 > 511) threshold_2 = 511;

        char buff[400];
        sprintf(buff, "Threshold1: %5.2f", threshold_1);
        add_line(buff);        
        sprintf(buff, "Threshold2: %5.2f", threshold_2);
        add_line(buff);        

    }
    

 if (wParam == 'H') begin_scan();
 
 if (wParam == 'J') laser_on();
 if (wParam == 'K') laser_off();
 
 if (wParam == 'L') abort_scan();
 if (wParam == 'M') scan_return();
    
    
    
    

}
/* End of process key down */


/* Called when a key is released */
void key_up( WPARAM wParam )
{
    keys[wParam % 256] = false;
}
/* End of process key up */




