

#include <stdio.h>

#include "types.h"
#include "display.h"
#include "camera.h"
#include "filters.h"
#include "move.h"
#include "bitmap.h"

extern color_type *temp_frame;
extern color_type *camera_buffer;

extern int size_x;
extern int size_y;
extern double threshold_1;
extern double threshold_2;

extern bool running;

bool scanning = 0;

double * scan_data;
int cur_scan = 0;

int max_scan = 640;

void show_scan( void )
{
    if (!camera_buffer) return;
    if (!scan_data) return;
  //  running = 0;
    
    double max = 0.001;
    
    
     for (int y = 0; y < size_y; y++)
        for (int x = 0; x < cur_scan; x++)   
            if (scan_data[x + y * max_scan] > max)
                max = scan_data[x + y * max_scan];

    
    
    for (int y = 0; y < size_y; y++)
    {
        for (int x = 0; x < size_x; x++)
        {
            int index = x *  cur_scan / size_x;
            
            if (index < 0) index = 0;
            if (index >= max_scan) index = max_scan-1;
            
            
            double  v = scan_data[index + y * max_scan] ;
            //if (v < threshold_1 / (double)255) v = 0;
            
            v -=  threshold_2 / (double)255;

            if (v < 0) v = 0;
            
            int c = (int)((v/max ) * 255);
            
            if (c > 255) c = 255;
            if (c < 0) c = 0;
            
            camera_buffer[x + y * size_x].r = c;
            camera_buffer[x + y * size_x].g = c;
            camera_buffer[x + y * size_x].b = c;
            
        }

    }
    
}


void scan_start( void )
{
    if (scan_data != NULL) free(scan_data);    

    scan_data = (double *) malloc(max_scan * size_y * sizeof(double));
                    
    if (scan_data == NULL)
    {            
        add_line("Unable to create scan_data");
        return;
    }        
    
}






void begin_scan( void )
{
    // get base frame
    camera_frame();

    // copy to temp frame
    memcpy(temp_frame, camera_buffer, DATA_SIZE);

    cur_scan = 0;

    scanning = true;
        
   laser_on();
        
  // Sleep(500);

  // move_motor_on(70);
}


void abort_scan( void )
{
    show_scan();
    save_to_disk();
   scanning = 0; 
   move_motor_off();
   laser_off();
}


void scan_stop( void )
{

    
    scanning = 0;   
   move_motor_off();
   laser_off();
    
    
    if (scan_data) free(scan_data);
}






void scan_return( void )
{
    move_motor_on(255);

    while (!get_limit());

    move_motor_off();
}




void get_scan_frame( void )
{
    if (!scanning) return;
        
    if (!temp_frame) return;
    if (!camera_buffer) return;
    if (!scan_data) return;
    
    if (cur_scan >= max_scan)
    {
        show_scan();
         save_to_disk();
         
        scanning = false;

        laser_off();

        move_motor_off();

        return;
    }


    char b[400];
    sprintf(b, "Line: %d  Percent: %4.2f", cur_scan, cur_scan/(double)max_scan * 100);
    add_line(b);
                   
    // get base frame
    camera_frame();
     
   //  display();
  
    move_forward(15);  // Now is a good time to move forward
        
    for(unsigned int i = 0; i < SIZE; i++)        
    {
        int r, g, b;
                
        double mag = color_size(camera_buffer[i]) - color_size(temp_frame[i]);
        
        color_type color;
        
        if (mag > threshold_1)
            color = black;
        else
            color = white;
        
        camera_buffer[i] = color;
    }    



    for (int y = 0; y < size_y; y++)
    {        
        int x = 0;
        bool found = 0;
            
         while (x < size_x && !found)
         {
             if (camera_buffer[x + y * size_x].r == 0)
             {
                    found = true;
                    scan_data[cur_scan + y * max_scan] = x / (double)size_x;

            }

           x++;           
        }   
        
        if (!found)
                scan_data[cur_scan + y * max_scan] = 0;        
    }
    
    cur_scan++;

    show_scan();
}





